
import matplotlib.pyplot as plt
import numpy as np
import random

from RCC9def import *

def get_random_color(pastel_factor = 0.5):
    return [(x+pastel_factor)/(1.0+pastel_factor) for x in [random.uniform(0,1.0) for i in [1,2,3]]]

def draw_bar():
    x = np.arange(len(key_list))

    fig, ax = plt.subplots()
    rects = ax.bar(x, dict.values(), align='center', color="magenta", alpha=0.5)

    ax.set_ylabel('occurrence')
    ax.set_title('RCC*-9 relations (3D features)')
    ax.set_xticks(x)
    ax.set_xticklabels(key_list)
    #ax.legend()

    def autolabel(rects):
        """Attach a text label above each bar in *rects*, displaying its height."""
        for rect in rects:
            height = rect.get_height()
            label="{:.2%}".format(height/number)
            ax.annotate('{}'.format(label),
                        xy=(rect.get_x() + rect.get_width() / 2, height),
                        xytext=(0, 3),  # 3 points vertical offset
                        textcoords="offset points",
                        ha='center', va='bottom')

    autolabel(rects)

    fig.tight_layout()
    plt.show()

def series_pol(npol, DX, OX):
#npol number of random polyhedrons that are produced
#DX maximum dimensions of polyhedrons
#OX maximum displacement of the origin
    list_paral = []
    for i in range(npol):
        DX = DX
        DY = DX
        DZ = DX
        dx = random.randint(-DX, DX)
        if dx == 0:
            dx = 1
        dy = random.randint(-DY, DY)
        if dy == 0:
            dy = 1
        dz = random.randint(-DZ, DZ)
        if dz == 0:
            dz = 1
        OX = OX
        OY = OX
        OZ = OX
        ox = random.randint(-OX, OX)
        oy = random.randint(-OY, OY)
        oz = random.randint(-OZ, OZ)

        paral = Parallelepiped(Point(ox, oy, oz), Vector(dx, 0, 0), Vector(0, dy, 0), Vector(0, 0, dz))
        list_paral = list_paral + [paral]
    return list_paral

def add_polygons(list_paral):
    list_poly=[]
    for paral in list_paral:
        polygons=paral.convex_polygons #a parallelepiped has 6 faces
        index=random.randint(0,5)
        list_poly.append(polygons[index])
    return list_poly
def add_segments(list_poly):
    list_seg=[]
    for i in range(len(list_poly)//2):
        pts=list_poly[i].points #a rectangular face has 4 points
        index=random.randint(0,3)
        p1=pts[index]
        i=i+1
        pts = list_poly[i].points  # a rectangular face has 4 points
        index = random.randint(0, 3)
        p2 = pts[index]
        seg=Segment(p1,p2)
        list_seg.append(seg)
    return list_seg
def find_poly_seg(list_paral):
    list_pts=()
    list_poly_seg=[]
    for paral in list_paral:
        polygons=paral.convex_polygons
        for poly in polygons:
            list_pts=list_pts+poly.points
    for i in range(len(list_paral)//2):
        p1,p2,p3=random.sample(list_pts,k=3)
        list_poly_seg.append(ConvexPolygon((p1,p2,p3)))
    for i in range(len(list_paral)//2):
        p1, p2 = random.sample(list_pts, k=2)
        list_poly_seg.append(Segment(p1, p2))
    return list_poly_seg

def experiment(npol):
    list_paral=series_pol(16,20,2) #series of big polyhedrons
    list_poly_seg = find_poly_seg(list_paral)  # derived polygons and segments
    list_paral = list_paral + list_poly_seg
    list_paral = list_paral + series_pol(8, 4, 8) #series of small polyhedrons

    r = Renderer()
    for paral in list_paral:
        color = tuple(get_random_color())
        r.add((paral, color, 2))
    r.show()

    list_rel=[]
    for paral1 in list_paral:
        for paral2 in list_paral:
            rel=RCC9(paral1,paral2)
            list_rel.append(rel)
    return list_rel

list_rel=experiment(npol=10)

dict=dict.fromkeys(key_list,0)
avg=dict.fromkeys(key_list)
number=len(list_rel)

for rel in list_rel:
    if len(rel) > 1 or rel==[]: #relation set is not pairwise disjoint or is not jointly exhaustive
        print(i, rel)
    dict[rel[0]]=dict[rel[0]]+1
print(dict)
for key in key_list:
    avg[key]=dict[key]/number
print(avg)
draw_bar()


