
import matplotlib.pyplot as plt
import numpy as np
import random

from RCC9def import *

def get_random_color(pastel_factor = 0.5):
    return [(x+pastel_factor)/(1.0+pastel_factor) for x in [random.uniform(0,1.0) for i in [1,2,3]]]

def draw_bar():
    x = np.arange(len(key_list))

    fig, ax = plt.subplots()
    rects = ax.bar(x, dict.values(), align='center', color="magenta", alpha=0.5)

    ax.set_ylabel('occurrence')
    ax.set_title('RCC*-9 relations (polyhedrons)')
    ax.set_xticks(x)
    ax.set_xticklabels(key_list)
    #ax.legend()

    def autolabel(rects):
        """Attach a text label above each bar in *rects*, displaying its height."""
        for rect in rects:
            height = rect.get_height()
            label="{:.2%}".format(height/number)
            ax.annotate('{}'.format(label),
                        xy=(rect.get_x() + rect.get_width() / 2, height),
                        xytext=(0, 3),  # 3 points vertical offset
                        textcoords="offset points",
                        ha='center', va='bottom')

    autolabel(rects)

    fig.tight_layout()
    plt.show()

def series_pol(npol, DX, OX):
#npol number of random polyhedrons that are produced
#DX maximum dimensions of polyhedrons
#OX maximum displacement of the origin
    list_paral = []
    for i in range(npol):
        DX = DX
        DY = DX
        DZ = DX
        dx = random.randint(-DX, DX)
        if dx == 0:
            dx = 1
        dy = random.randint(-DY, DY)
        if dy == 0:
            dy = 1
        dz = random.randint(-DZ, DZ)
        if dz == 0:
            dz = 1
        OX = OX
        OY = OX
        OZ = OX
        ox = random.randint(-OX, OX)
        oy = random.randint(-OY, OY)
        oz = random.randint(-OZ, OZ)

        paral = Parallelepiped(Point(ox, oy, oz), Vector(dx, 0, 0), Vector(0, dy, 0), Vector(0, 0, dz))
        list_paral = list_paral + [paral]
    return list_paral

def experiment(npol):
    list_paral=series_pol(20,20,2) #series of big poly near the origin
    list_paral=list_paral+series_pol(10,4,8) #series of small poly further away from the origin

    r = Renderer()
    for paral in list_paral:
        color = tuple(get_random_color())
        r.add((paral, color, 2))
    r.show()

    list_rel=[]
    for paral1 in list_paral:
        for paral2 in list_paral:
            rel=RCC9(paral1,paral2)
            list_rel.append(rel)
    return list_rel

list_rel=experiment(npol=10)

dict=dict.fromkeys(key_list,0)
avg=dict.fromkeys(key_list)
number=len(list_rel)

for rel in list_rel:
    if len(rel) > 1 or rel==[]: #relation set is not pairwise disjoint or is not jointly exhaustive
        print(i, rel)
    dict[rel[0]]=dict[rel[0]]+1
print(dict)
for key in key_list:
    avg[key]=dict[key]/number
print(avg)
draw_bar()


