
import matplotlib.pyplot as plt
import numpy as np
import random

from RCC9def import *

U={'DC', 'EC', 'PO', 'TPP', 'NTPP', 'TPPI', 'NTPPI', 'EQ', 'CR'}
T = {}

#table for simple regions
T['DC'] = {'DC': U,
           'EC': {'DC', 'EC', 'PO', 'TPP', 'NTPP'},
           'PO': {'DC', 'EC', 'PO', 'TPP', 'NTPP'},
           'TPP': {'DC', 'EC', 'PO', 'TPP', 'NTPP'},
           'NTPP': {'DC', 'EC', 'PO', 'TPP', 'NTPP'},
           'TPPI': {'DC'},
           'NTPPI': {'DC'},
           'EQ': {'DC'},
           'CR': set()}
T['EC'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI'},
           'EC': {'DC', 'EC', 'PO', 'TPP', 'TPPI', 'EQ'},
           'PO': {'DC', 'EC', 'PO', 'TPP', 'NTPP'},
           'TPP': {'EC', 'PO', 'TPP', 'NTPP'},
           'NTPP': {'PO', 'TPP', 'NTPP'},
           'TPPI': {'DC', 'EC'},
           'NTPPI': {'DC'},
           'EQ': {'EC'},
           'CR': set()}
T['PO'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI'},
           'EC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI'},
           'PO': U,
           'TPP': {'PO', 'TPP', 'NTPP'},
           'NTPP': {'PO', 'TPP', 'NTPP'},
           'TPPI': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI'},
           'NTPPI': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI'},
           'EQ': {'PO'},
           'CR': set()}
T['TPP'] = {'DC': {'DC'},
           'EC': {'DC', 'EC'},
           'PO': {'DC', 'EC', 'PO', 'TPP', 'NTPP'},
           'TPP': {'TPP', 'NTPP'},
           'NTPP': {'NTPP'},
           'TPPI': {'DC', 'EC', 'PO', 'TPP', 'TPPI', 'EQ'},
           'NTPPI': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI'},
           'EQ': {'TPP'},
           'CR': set()}
T['NTPP'] = {'DC': {'DC'},
           'EC': {'DC'},
           'PO': {'DC', 'EC', 'PO', 'TPP', 'NTPP'},
           'TPP': {'NTPP'},
           'NTPP': {'NTPP'},
           'TPPI': {'DC', 'EC', 'PO', 'TPP', 'NTPP'},
           'NTPPI': U,
           'EQ': {'NTPP'},
           'CR': set()}
T['TPPI'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI'},
           'EC': {'EC', 'PO', 'TPPI', 'NTPPI'},
           'PO': {'PO', 'TPPI', 'NTPPI'},
           'TPP': {'PO', 'TPP', 'TPPI', 'EQ'},
           'NTPP': {'PO', 'TPP', 'NTPP'},
           'TPPI': {'TPPI', 'NTPPI'},
           'NTPPI': {'NTPPI'},
           'EQ': {'TPPI'},
           'CR': set()}
T['NTPPI'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI'},
           'EC': {'PO', 'TPPI', 'NTPPI'},
           'PO': {'PO', 'TPPI', 'NTPPI'},
           'TPP': {'PO', 'TPPI', 'NTPPI'},
           'NTPP': {'PO', 'TPP', 'NTPP', 'TPPI', 'NTPPI', 'EQ'},
           'TPPI': {'NTPPI'},
           'NTPPI': {'NTPPI'},
           'EQ': {'NTPPI'},
           'CR': set()}
T['EQ'] = {'DC': {'DC'},
           'EC': {'EC'},
           'PO': {'PO'},
           'TPP': {'TPP'},
           'NTPP': {'NTPP'},
           'TPPI': {'TPPI'},
           'NTPPI': {'NTPPI'},
           'EQ': {'EQ'},
           'CR': set()}
T['CR'] = {'DC': set(),
           'EC': set(),
           'PO': set(),
           'TPP': set(),
           'NTPP': set(),
           'TPPI': set(),
           'NTPPI': set(),
           'EQ': set(),
           'CR': set()}

def compare (comp,T):
    for i in key_list:
        for j in key_list:
            if comp[i][j]==T[i][j]:
                pass
            else:
                print('row:',i)
                print('col:',j)
                print('comp=',comp[i][j])
                print('T=',T[i][j])
                print('miss: ', T[i][j].difference(comp[i][j]),'\n', 'found:',comp[i][j].difference(T[i][j]))
                print()
                
def get_random_color(pastel_factor = 0.5):
    return [(x+pastel_factor)/(1.0+pastel_factor) for x in [random.uniform(0,1.0) for i in [1,2,3]]]


def series_pol(npol, DX, OX):
#npol number of random polyhedrons that are produced
#DX maximum dimensions of polyhedrons
#OX maximum displacement of the origin
    list_paral = []
    for i in range(npol):
        DX = DX
        DY = DX
        DZ = DX
        dx = random.randint(-DX, DX)
        if dx == 0:
            dx = 1
        dy = random.randint(-DY, DY)
        if dy == 0:
            dy = 1
        dz = random.randint(-DZ, DZ)
        if dz == 0:
            dz = 1
        OX = OX
        OY = OX
        OZ = OX
        ox = random.randint(-OX, OX)
        oy = random.randint(-OY, OY)
        oz = random.randint(-OZ, OZ)

        paral = Parallelepiped(Point(ox, oy, oz), Vector(dx, 0, 0), Vector(0, dy, 0), Vector(0, 0, dz))
        list_paral = list_paral + [paral]
    return list_paral

def build_tablecod(list_paral):
    n=len(list_paral)
    tablecod=[[0]*n for i in range(n)]
    for i in range(n):
        for j in range(n):
            rel=RCC9(list_paral[i],list_paral[j])
            if len(rel) > 1 or rel == []:
                print(i, rel)
            tablecod[i][j]=rel[0]
    return tablecod

def experiment(npol):
    list_paral=series_pol(npol,20,2) #series of big poly near the origin
    list_paral=list_paral+series_pol(npol//2,4,8) #series of small poly further away from the origin

    r = Renderer()
    for paral in list_paral:
        color = tuple(get_random_color())
        r.add((paral, color, 2))
    r.show()

    tablecod=build_tablecod(list_paral)
    return tablecod

tablecod=experiment(npol=10)

comp=dict.fromkeys(key_list,{})
for key in key_list:
    comp[key]=dict.fromkeys(key_list,set())

n=len(tablecod)
for i in range(n):
    for j in range(n):
        for k in range(n):
            r1=tablecod[i][j]
            r2=tablecod[j][k]
            r3=tablecod[i][k]
            comp[r1][r2]=comp[r1][r2].union({r3})

compare(comp,T)


