from osgeo import ogr
from visuallayer import *
import matplotlib.pyplot as plt
import random

U={'DC', 'EC', 'PO', 'TPP', 'NTPP', 'TPPI', 'NTPPI', 'EQ', 'CR'}
T = {}

#table for complex regions
T['DC'] = {'DC': U,
           'EC': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'PO': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'TPP': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'NTPP': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'TPPI': {'DC'},
           'NTPPI': {'DC'},
           'EQ': {'DC'},
           'CR': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'}}
T['EC'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'EC': {'DC', 'EC', 'PO', 'TPP', 'TPPI', 'EQ', 'CR'},
           'PO': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'TPP': {'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'NTPP': {'PO', 'TPP', 'NTPP', 'CR'},
           'TPPI': {'DC', 'EC'},
           'NTPPI': {'DC'},
           'EQ': {'EC'},
           'CR': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'}}
T['PO'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'EC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'PO': U,
           'TPP': {'PO', 'TPP', 'NTPP', 'CR'},
           'NTPP': {'PO', 'TPP', 'NTPP', 'CR'},
           'TPPI': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'NTPPI': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'EQ': {'PO'},
           'CR': U-{'EQ'}}
T['TPP'] = {'DC': {'DC'},
           'EC': {'DC', 'EC'},
           'PO': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'TPP': {'TPP', 'NTPP'},
           'NTPP': {'NTPP'},
           'TPPI': {'DC', 'EC', 'PO', 'TPP', 'TPPI', 'EQ', 'CR'},
           'NTPPI': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'EQ': {'TPP'},
           'CR': {'DC', 'EC', 'PO', 'NTPP', 'CR'}}
T['NTPP'] = {'DC': {'DC'},
           'EC': {'DC'},
           'PO': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'TPP': {'NTPP'},
           'NTPP': {'NTPP'},
           'TPPI': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'},
           'NTPPI': U,
           'EQ': {'NTPP'},
           'CR': {'DC', 'EC', 'PO', 'TPP', 'NTPP', 'CR'}}
T['TPPI'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'EC': {'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'PO': {'PO', 'TPPI', 'NTPPI', 'CR'},
           'TPP': {'PO', 'TPP', 'TPPI', 'EQ'},
           'NTPP': {'PO', 'TPP', 'NTPP', 'CR'},
           'TPPI': {'TPPI', 'NTPPI'},
           'NTPPI': {'NTPPI'},
           'EQ': {'TPPI'},
           'CR': {'PO', 'TPPI', 'NTPPI', 'CR'}}
T['NTPPI'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'EC': {'PO', 'TPPI', 'NTPPI', 'CR'},
           'PO': {'PO', 'TPPI', 'NTPPI', 'CR'},
           'TPP': {'PO', 'TPPI', 'NTPPI', 'CR'},
           'NTPP': {'PO', 'TPP', 'NTPP', 'TPPI', 'NTPPI', 'EQ', 'CR'},
           'TPPI': {'NTPPI'},
           'NTPPI': {'NTPPI'},
           'EQ': {'NTPPI'},
           'CR': {'PO', 'TPPI', 'NTPPI', 'CR'}}
T['EQ'] = {'DC': {'DC'},
           'EC': {'EC'},
           'PO': {'PO'},
           'TPP': {'TPP'},
           'NTPP': {'NTPP'},
           'TPPI': {'TPPI'},
           'NTPPI': {'NTPPI'},
           'EQ': {'EQ'},
           'CR': {'CR'}}
T['CR'] = {'DC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'EC': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'PO': U-{'EQ'},
           'TPP': {'PO', 'TPP', 'NTPP', 'CR'},
           'NTPP': {'PO', 'TPP', 'NTPP', 'CR'},
           'TPPI': {'DC', 'EC', 'PO', 'NTPPI', 'CR'},
           'NTPPI': {'DC', 'EC', 'PO', 'TPPI', 'NTPPI', 'CR'},
           'EQ': {'CR'},
           'CR': U}


def compare (comp,T):
    for i in key_list:
        for j in key_list:
            if comp[i][j]==T[i][j]:
                pass
            else:
                print('row:',i)
                print('col:',j)
                print('comp=',comp[i][j])
                print('T=',T[i][j])
                print('miss: ', T[i][j].difference(comp[i][j]),'\n', 'found:',comp[i][j].difference(T[i][j]))
                print()

def create_twosquares(tx=0,ty=0,zx=1,zy=1): #tx ty translate, zx zy zoom
    ring5 = ogr.Geometry(ogr.wkbLinearRing) #outer ring
    ring5.AddPoint(0*zx+tx, 0*zy+ty)
    ring5.AddPoint(3*zx+tx, 0*zy+ty)
    ring5.AddPoint(3*zx+tx, 3*zy+ty)
    ring5.AddPoint(0*zx+tx, 3*zy+ty)
    ring5.AddPoint(0*zx+tx, 0*zy+ty)

    outer = ogr.Geometry(ogr.wkbLinearRing) #disconnected component ring
    outer.AddPoint(5*zx+tx, 1*zy+ty)
    outer.AddPoint(6*zx+tx, 1*zy+ty)
    outer.AddPoint(6*zx+tx, 2*zy+ty)
    outer.AddPoint(5*zx+tx, 2*zy+ty)
    outer.AddPoint(5*zx+tx, 1*zy+ty)

    poly = ogr.Geometry(ogr.wkbPolygon)
    poly.AddGeometry(ring5)
    comp = ogr.Geometry(ogr.wkbPolygon) #disconnected component
    comp.AddGeometry(outer)

    mpoly = ogr.Geometry(ogr.wkbMultiPolygon)
    mpoly.AddGeometry(poly)
    mpoly.AddGeometry(comp)
    #print (mpoly.ExportToWkt())
    return mpoly
def create_twosquaresinv(tx=0,ty=0,zx=1,zy=1): #tx ty translate, zx zy zoom
    ring5 = ogr.Geometry(ogr.wkbLinearRing) #outer ring
    ring5.AddPoint(0*zx+tx, 0*zy+ty)
    ring5.AddPoint(3*zx+tx, 0*zy+ty)
    ring5.AddPoint(3*zx+tx, 3*zy+ty)
    ring5.AddPoint(0*zx+tx, 3*zy+ty)
    ring5.AddPoint(0*zx+tx, 0*zy+ty)

    outer = ogr.Geometry(ogr.wkbLinearRing) #disconnected component ring
    outer.AddPoint(-3*zx+tx, 1*zy+ty)
    outer.AddPoint(-2*zx+tx, 1*zy+ty)
    outer.AddPoint(-2*zx+tx, 2*zy+ty)
    outer.AddPoint(-3*zx+tx, 2*zy+ty)
    outer.AddPoint(-3*zx+tx, 1*zy+ty)

    poly = ogr.Geometry(ogr.wkbPolygon)
    poly.AddGeometry(ring5)
    comp = ogr.Geometry(ogr.wkbPolygon) #disconnected component
    comp.AddGeometry(outer)

    mpoly = ogr.Geometry(ogr.wkbMultiPolygon)
    mpoly.AddGeometry(poly)
    mpoly.AddGeometry(comp)
    #print (mpoly.ExportToWkt())
    return mpoly

def twosquares_random(list_mpoly, n,traslx,trasly,maxzoom,minzoom):
    for i in range(int(n/2)):
        tx=random.randint(-traslx,traslx)
        ty=random.randint(-trasly,trasly)
        zx=minzoom+random.randint(1,maxzoom)
        zy=minzoom+random.randint(1,maxzoom)
        mpoly=create_twosquares(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
    for i in range(int(n/2)):
        tx=random.randint(-traslx,traslx)
        ty=random.randint(-trasly,trasly)
        zx=minzoom+random.randint(1,maxzoom)
        zy=minzoom+random.randint(1,maxzoom)
        mpoly=create_twosquaresinv(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
    return list_mpoly

def create_mpoly(tx=0,ty=0,zx=1,zy=1): #tx ty translate, zx zy zoom
    ring5 = ogr.Geometry(ogr.wkbLinearRing) #outer ring
    ring5.AddPoint(0*zx+tx, 0*zy+ty)
    ring5.AddPoint(5*zx+tx, 0*zy+ty)
    ring5.AddPoint(5*zx+tx, 5*zy+ty)
    ring5.AddPoint(0*zx+tx, 4*zy+ty)
    ring5.AddPoint(0*zx+tx, 0*zy+ty)

    hole0 = ogr.Geometry(ogr.wkbLinearRing) #hole
    hole0.AddPoint(1*zx+tx, 1*zy+ty)
    hole0.AddPoint(3*zx+tx, 1*zy+ty)
    hole0.AddPoint(3*zx+tx, 3*zy+ty)
    hole0.AddPoint(1*zx+tx, 3*zy+ty)
    hole0.AddPoint(1*zx+tx, 1*zy+ty)

    outer = ogr.Geometry(ogr.wkbLinearRing) #disconnected component ring
    outer.AddPoint(0*zx+tx, -1*zy+ty)
    outer.AddPoint(0*zx+tx, -2*zy+ty)
    outer.AddPoint(2*zx+tx, -2*zy+ty)
    outer.AddPoint(1*zx+tx, -1*zy+ty)
    outer.AddPoint(0*zx+tx, -1*zy+ty)
    poly = ogr.Geometry(ogr.wkbPolygon)
    poly.AddGeometry(ring5)
    poly.AddGeometry(hole0)

    comp = ogr.Geometry(ogr.wkbPolygon) #disconnected component
    comp.AddGeometry(outer)

    mpoly = ogr.Geometry(ogr.wkbMultiPolygon)
    mpoly.AddGeometry(poly)
    mpoly.AddGeometry(comp)
    #print (mpoly.ExportToWkt())
    return mpoly
def mpoly_random(list_mpoly,n,traslx,trasly,maxzoom,minzoom):
    for i in range(n):
        tx=random.randint(-traslx,traslx)
        ty=random.randint(-trasly,trasly)
        zx=minzoom+maxzoom*random.random()
        zy=minzoom+maxzoom*random.random()
        mpoly=create_mpoly(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
    return list_mpoly

def create_poly(tx=0,ty=0,zx=1,zy=1): #tx ty translate, zx zy zoom
    ring5 = ogr.Geometry(ogr.wkbLinearRing) #outer ring
    ring5.AddPoint(0*zx+tx, 0*zy+ty)
    ring5.AddPoint(9*zx+tx, 0*zy+ty)
    ring5.AddPoint(9*zx+tx, 5*zy+ty)
    ring5.AddPoint(0*zx+tx, 6*zy+ty)
    ring5.AddPoint(0*zx+tx, 0*zy+ty)
    poly = ogr.Geometry(ogr.wkbPolygon)
    poly.AddGeometry(ring5)
    return poly
def create_square(tx=0,ty=0,zx=1,zy=1): #tx ty translate, zx zy zoom
    ring5 = ogr.Geometry(ogr.wkbLinearRing) #outer ring
    ring5.AddPoint(0*zx+tx, 1*zy+ty)
    ring5.AddPoint(1*zx+tx, 1*zy+ty)
    ring5.AddPoint(1*zx+tx, 2*zy+ty)
    ring5.AddPoint(0*zx+tx, 2*zy+ty)
    ring5.AddPoint(0*zx+tx, 1*zy+ty)
    poly = ogr.Geometry(ogr.wkbPolygon)
    poly.AddGeometry(ring5)
    return poly

def simple_random(list_mpoly, n,traslx,trasly,maxzoom,minzoom):
    for i in range(int(n/2)):
        tx=random.randint(-traslx,traslx)
        ty=random.randint(-trasly,trasly)
        zx=minzoom+random.randint(1,maxzoom)
        zy=minzoom+random.randint(1,maxzoom)
        mpoly=create_poly(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
    for i in range(int(n/2)):
        tx=random.randint(-traslx,traslx)
        ty=random.randint(-trasly,trasly)
        zx=minzoom+random.randint(1,maxzoom)
        zy=minzoom+random.randint(1,maxzoom)
        mpoly=create_square(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
    return list_mpoly

def particular_cases(list_mpoly, n,traslx,trasly,maxzoom,minzoom):
    for i in range(n):
        tx = random.randint(-traslx, traslx)
        ty = random.randint(-trasly, trasly)
        zx = minzoom + random.randint(1, maxzoom)
        zy = minzoom + random.randint(1, maxzoom)
        mpoly=create_twosquares(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
        mpoly=create_square(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
        mpoly=create_poly(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
        mpoly=create_twosquaresinv(tx,ty,zx,zy)
        list_mpoly.append(mpoly)
        #draw(list_mpoly)
    return list_mpoly

from shapely import wkt
def relate(geom1,geom2):
    st1=geom1.ExportToWkt()
    st2=geom2.ExportToWkt()
    obj1=wkt.loads(st1)
    obj2 = wkt.loads(st2)
    pattern=obj1.relate(obj2)
    return pattern


import re
def cod_RCC9(patternDE9IM):
    pattern9IM=""
    for ch in patternDE9IM:
        if ch=="F":
            pattern9IM=pattern9IM+ch
        else:
            pattern9IM=pattern9IM+'T'
    cod = []
    DC = re.compile('FF.FF...')
    if re.match(DC, pattern9IM):
        cod.append('DC')
    EC1 = re.compile('FTT...T..')
    EC2 = re.compile('F.TT..T..')
    EC3 = re.compile('F.T.T.T..')
    if re.match(EC1, pattern9IM) or re.match(EC2, pattern9IM) or re.match(EC3, pattern9IM):
        cod.append('EC')
    NTPP = re.compile('.FF.FF...')
    if re.match(NTPP, pattern9IM):
        cod.append('NTPP')
    TPP1 = re.compile('.TF..F...')
    TPP2 = re.compile('..F.TF...')
    TPP3 = re.compile('TFFFTFFFT')
    if re.match(TPP1, pattern9IM) or re.match(TPP2, pattern9IM) and not re.match(TPP3, pattern9IM):
        cod.append('TPP')
    CR1 = re.compile('T.TFFTT..')
    CR2 = re.compile('TFT.F.TT.')
    if re.match(CR1, pattern9IM) or re.match(CR2, pattern9IM):
        cod.append('CR')
    PO1 = re.compile('TTTT..T..')
    PO2 = re.compile('T.T.T.T..')
    if re.match(PO1, pattern9IM) or re.match(PO2, pattern9IM):
        cod.append('PO')
    NTPPI = re.compile('...FF.FF.')
    if re.match(NTPPI, pattern9IM):
        cod.append('NTPPI')
    TPPI1 = re.compile('...T..FF.')
    TPPI2 = re.compile('....T.FF.')
    TPPI3 = re.compile('TFFFTFFFT')
    if re.match(TPPI1, pattern9IM) or re.match(TPPI2, pattern9IM) and not re.match(TPPI3, pattern9IM):
        cod.append('TPPI')
    EQ = re.compile('TFFFTFFFT')
    if re.match(EQ, pattern9IM):
        cod.append('EQ')
    return cod
'''
DC(x,y)	Relate(x,y,"FF*FF****")
EC(x,y)	Relate(x,y,"FTT***T**") OR      
        Relate(x,y,"F*TT**T**") OR
        Relate(x,y,"F*T*T*T**")
NTPP(x,y)	Relate(x,y,"*FF*FF***")
TPP(x,y)	Relate(x,y,"*TF**F***") OR
            Relate(x,y,"**F*TF***") AND
            NOT Relate(x,y,"TFFFTFFFT")
CR(x,y)	Relate(x,y,"T*TFFTTT*") OR      
        Relate(x,y,"TFT*FTTT*")
PO(x,y)	Relate(x,y,"TTTT**T**") OR      
        Relate(x,y,"T*T*T*T**")
NTPPi(x,y)	Relate(x,y,"***FF*FF*")
TPPi(x,y)	Relate(x,y,"***T**FF*") OR 
            Relate(x,y,"****T*FF*") AND
            NOT Relate(x,y,"TFFFTFFFT")
EQ(x,y)	Relate(x,y,"TFFFTFFFT")
'''

def draw(list_mpoly):
    for mpoly in list_mpoly:
        color = get_random_color()
        plot_geometry(mpoly, edgecolor="grey", fillcolor=color, color=color,
                holeedgecolor='grey', holefillcolor='white', fill=True)
    plt.show()

def build_tablecod(n):
    table=[[0]*n for i in range(n)]
    for i in range(n):
        for j in range(n):
            pattern=relate(list_mpoly[i],list_mpoly[j])
            table[i][j]=pattern

    tablecod=[[0]*n for i in range(n)]
    for i in range(n):
        for j in range(n):
            rel = cod_RCC9(table[i][j])
            if len(rel) > 1 or rel == []:
                print(i, rel)
            tablecod[i][j]=rel[0]
    return tablecod

key_list = ['DC', 'EC', 'PO', 'TPP', 'NTPP', 'TPPI', 'NTPPI', 'EQ', 'CR']
comp=dict.fromkeys(key_list,{})
for key in key_list:
    comp[key]=dict.fromkeys(key_list,set())

for times in range(1):
    list_mpoly=[]
    list_mpoly=simple_random([],100,5,5,3,0) #100,5,5,3,0 gives sort of equiprobable set of relations
    addlist=twosquares_random([],100,15,15,4,0)
    list_mpoly = list_mpoly + addlist
    addlist=mpoly_random([],100,10,10,4,0.5)
    list_mpoly=list_mpoly+addlist
    addlist=particular_cases([],10,15,15,4,0)
    list_mpoly=list_mpoly+addlist
    #draw(list_mpoly)
    n=len(list_mpoly)
    tablecod=build_tablecod(n)
    dict = dict.fromkeys(key_list, 0)
    for i in range(n):
        for j in range(n):
            r=tablecod[i][j]
            dict[r] = dict[r] + 1
    print(dict)
    #print(tablecod)
    for i in range(n):
        for j in range(n):
            for k in range(n):
                r1=tablecod[i][j]
                r2=tablecod[j][k]
                r3=tablecod[i][k]
                comp[r1][r2]=comp[r1][r2].union({r3})
    
compare(comp,T)

#impossible cases for region-region (complex):
#TPP + CR = TPP
#CR + TPPI = TPPI